﻿' Copyright © exc-jdbi 2017
'
' © NCS 2017
'
' © NCS is Free and Opensource Software
'
'www.vb-paradise.de/index.php/User/23082-exc-jdbi/
'
'github.com/exc-jdbi


Option Strict On
Option Explicit On

'Die Klasse NCS ist das Naturtalent, als Integer-Variante
'Sie convertiert natürliche Zahlen (exkl. 0) gesplittet in einer Integer-Array 
'von einer Basis ins Andere, und das sehr zügig.

'Wichtig: Es sind keine Exception-Behandlungen vorhanden


''' <summary>
''' © NCS 2017 - Numeral System Converter.
''' Created by © exc-jdbi 2017
''' </summary>
Public Class NSC
    ''' <summary>
    ''' Converts an Integer-Array with 'Natural Numbers' from one base to another.
    ''' Important: There are no exception treatments. 
    ''' Created by @ exc-jdbi 2017.
    ''' </summary>
    ''' <param name="input">Initial values</param>
    ''' <param name="startBase">Starting base</param>
    ''' <param name="targetBase">Target base</param>
    ''' <returns>Returns a list with the new values</returns>
    ''' <history>Created by © exc-jdbi 2017</history>
    Public Function convert(ByVal input() As Integer, ByVal startBase As Integer, ByVal targetBase As Integer) As Integer()
        convert = Nothing
        'Prüfen der vorhandenen Werte, ansonsten Return Nothing
        If (input.Length > 0) AndAlso (CheckEmptyLess0(input)) AndAlso (startBase > 1) AndAlso (targetBase > 1) Then
            'Variable die sicherstellt, dass die Schleife verlassen wird,
            'wenn alle ipt-Werte '0'  sind
            Dim ext As Boolean = True

            'Zwei Variablen eine für das Resultat und eine für das Zwischenergebnis
            Dim r As Integer = 0, rT As Integer = 0

            'Kopie erstellen von input. Ist nötig, da obwohl bei der Übergabe Call by Value besteht,
            'sich die Variable input wie call by Reference verhält. Integer-Array ist eben ein Object.
            Dim ipt() As Integer = DirectCast(input.Clone, Integer())

            'Kapazität berechnen
            Dim cap As Integer = CInt(input.Length * Math.Log(startBase) / Math.Log(targetBase)) + 1

            'Übergabevariable instanzieren
            Dim lst As New List(Of Integer)(cap)

            'Die For-Schleife iteriert durch die ganze Integer-Array. 
            'Zwei Schritte werden gemacht:
            'Erster Schritt:    rT berechnen nach der entsprechenden Formel (Zwischenschritt)
            'Zweiter Schritt:   den neuen Wert ausrechnen nach entsprechender Formel
            'rT wird an r übergeben, damit das Zwischenergebnis immer um i-1 zurückhinkt, sobald die 
            'For-Schleife wieder von vorne anfängt.
            'Zum Schluss eine kleine Prüfung, ob der neue Wert in ipt(i) grösser 0 ist. 
            '>>> Sollte durchgehen 0 vorhanden sein wird 'ext' auf False bleiben und somit
            '>>> wird die while-Schleife am Schluss verlassen.

            'In der Whileschleife werden r (auf 0) und ext (zu False) immer wieder zurückgesetzt
            '- Der errechnete Endwert wird in der ListOfT festgehalten
            '- ext wird geprüft ob False. Wenn False wird die Schleife verlassen.
            While (ext)
                r = 0 : ext = False
                For i As Integer = 0 To ipt.Length - 1
                    rT = ((startBase * r) + ipt(i)) Mod targetBase
                    ipt(i) = ((startBase * r) + ipt(i)) \ targetBase
                    r = rT
                    If ipt(i) > 0 Then ext = True
                Next
                lst.Insert(0, r)
            End While
            'Das Zurücksetzen des ListOfT-Kapazitätspeichers ist nicht mehr nötig, 
            'da oben sehr genaue cap-Berechnung (innerhalb der Schwellgrenze)
            'lst.TrimExcess() 

            'Die ListOfT übergeben
            Return lst.ToArray
        End If
    End Function
    'Prüft den Eingangswert auf Empty oder Zahlen kleiner 0
    <System.Diagnostics.DebuggerStepThrough()> _
    Private Function CheckEmptyLess0(ByVal inpt() As Integer) As Boolean
        If Not inpt.Where(Function(x) (x < 1)).Count = inpt.Length Then
            If inpt.Where(Function(x) (x < 0)).Count = 0 Then
                Return True
            End If
        End If
        Return False
    End Function
End Class



